//
//  Scrippet.js
//  Snippets
//
//  Created by Vladimir Prudnikov on 04/02/2010.
//  Copyright 2010 All rights reserved.
//

/* Scrippet object is a singleton */
var Scrippet = null;
if (Scrippet == null)
    Scrippet = function() {
        /* Define a class to represent Cocoa library items in HTML */
        function SnippetObject(data) {
            // Initialize snippet attributes
            var uid = data.uid || "";
            var name = data.name || "";
            var author = data.author || "";
            var desc = data.desc || "";
            var relatedURL = data.relatedURL || "";
            var sourceCode = data.sourceCode || "";
            var highlight = data.highlight || "";
            var exportFilename = data.exportFilename || "";
            var labels = data.labels ? data.labels.split(",") : [];
            
            // Modification flags for Cocoa app
            var changeMask = data.changeMask || 0;
            
            // Define modification constants
            var CHANGE_MASK_NAME = 1 << 0;
            var CHANGE_MASK_AUTHOR = 1 << 1;
            var CHANGE_MASK_DESC = 1 << 2;
            var CHANGE_MASK_RELATED_URL = 1 << 3;
            var CHANGE_MASK_SOURCE_CODE = 1 << 4;
            var CHANGE_MASK_HIGHLIGHT = 1 << 5;
            var CHANGE_MASK_LABELS = 1 << 6;
            var CHANGE_MASK_EXPORT_FILENAME = 1 << 7;
            
            // Make a dictionary of custom properties
            var customProperties = {};
            var len = data.customKeys ? data.customKeys.length : 0;
            for (var i = 0; i < len; i++) {
                var key = data.customKeys[i];
                var value = data.customValues[i];
                if (key && value)
                    customProperties[key] = value;
            }
            
            // Read-only reader for modification flag
            this.isChanged = function () { return changeMask > 0; };
            this.getChangeMask = function () { return changeMask; };
            
            // Getters
            this.getUID = function () { return uid; };
            this.getName = function () { return name; };
            this.getAuthor = function () { return author; };
            this.getDescription = function () { return desc; };
            this.getRelatedURL = function () { return relatedURL; };
            this.getSourceCode = function () { return sourceCode; };
            this.getHighlight = function () { return highlight; };
            this.getLabels = function () { return labels; };
            this.getExportFilename = function () { return exportFilename; };
            
            // Setters
            this.setName = function (value) { if (name == value) return; name = value; changeMask |= CHANGE_MASK_NAME; };
            this.setAuthor = function (value) { if (author == value) return; author = value; changeMask |= CHANGE_MASK_AUTHOR; };
            this.setDescription = function (value) { if (desc == value) return; desc = value; changeMask |= CHANGE_MASK_DESC; };
            this.setRelatedURL = function (value) { if (relatedURL == value) return; relatedURL = value; changeMask |= CHANGE_MASK_RELATED_URL; };
            this.setSourceCode = function (value) { if (sourceCode == value) return; sourceCode = value; changeMask |= CHANGE_MASK_SOURCE_CODE; };
            this.setHighlight = function (value) { if (highlight == value) return; highlight = value; changeMask |= CHANGE_MASK_HIGHLIGHT; };
            this.setLabels = function (value) { if (labels.join(",") == value.join(",")) return; labels = value; changeMask |= CHANGE_MASK_LABELS; };
            this.setExportFilename = function (value) { if (exportFilename == value) return; exportFilename = value; changeMask |= CHANGE_MASK_EXPORT_FILENAME; };
            
            // Custom properties will be passed into Shell script as environment variables.
            // This way, we can have separate properties for every snippet during script execution.
            this.getProperty = function (name) { return customProperties[name]; };
            this.setProperty = function (name, value) { customProperties[name] = value; };
            
            // Flexible label management
            this.addLabel = function (label) {
                var uppercasedLabel = label.toUpperCase();
                var index = -1;
                for (var i = 0; i < labels.length; i++)
                    if (labels[i].toUpperCase() == uppercasedLabel) {
                        index = i;
                        break;
                    }
                if (index < 0) {
                    labels.push(label);
                    changeMask |= CHANGE_MASK_LABELS;
                }
            };
            this.removeLabel = function (label) {
                var uppercasedLabel = label.toUpperCase();
                var index = -1;
                for (var i = 0; i < labels.length; i++)
                    if (labels[i].toUpperCase() == uppercasedLabel) {
                        index = i;
                        break;
                    }
                if (index < 0) return;
                
                labels = labels.slice(0, index).concat(labels.slice(index + 1));
                changeMask |= CHANGE_MASK_LABELS;
            };
            
            // This method generates key-value dictionary to pass back into Cocoa app
            this._getCocoaDictionary = function () {
                
                // The only way to pass dictionary back is to split it into keys and values
                var customKeys = [];
                var customValues = [];
                for (var key in customProperties)
                {
                    customKeys.push(key);
                    customValues.push(customProperties[key]);
                }
                
                return {
                    uid: uid,
                    name: name,
                    author: author,
                    desc: desc,
                    relatedURL: relatedURL,
                    sourceCode: sourceCode,
                    highlight: highlight,
                    labels: labels,
                    exportFilename: exportFilename,
                    changeMask: changeMask,
                    customKeys: customKeys,
                    customValues: customValues
                };
            };

            // Returns dictionary of attributes for using in JavaScript code
            this.getAttributes = function () {
                return {
                    uid: uid,
                    name: name,
                    author: author,
                    desc: desc,
                    relatedURL: relatedURL,
                    sourceCode: sourceCode,
                    highlight: highlight,
                    labels: labels,
                    exportFilename: exportFilename
                };
            };
        };
        
        // Private members
        var instance = null;
        var snippets = [];
        var globalProperties = {};
        var clipboardProxy = null;
        var scrippetSheet = null;
        
        function ScrippetSingleton() {
            
            // Properties are passed between Scrippet steps:
            // HTML pages request them as object values by name,
            // Scripts access them as environment variables
            this.getProperty = function (name) { return globalProperties[name]; };
            
            // String is a preferred format for property Values because
            // all environment variables are casted to the plain text
            this.setProperty = function (name, value) { globalProperties[name] = value; };
            
            // Persistent properties allow to store auth tokens,
            // account names, preferences etc. between user sessions
            this.loadPersistentItem = function (name) { return localStorage.getItem(name); };
            
            // Persistence is implemented using HTML5 local
            // database APIs that are managed by WebKit itself
            this.storePersistentItem = function (name, value) { localStorage.setItem(name, value); };
            
            // Scrippet can hold collection of snippet objects;
            // changes in these objects will be reflected in the App
            this.getSnippets = function () { return snippets; };
            
            // This factory method creates new snippet object that
            // will be added into the application Library
            this.createSnippet = function () {
                var snippet = new SnippetObject();
                snippets.push(snippet);
                return snippet;
            };
            
            // This callback is to be called after data is loaded
            this.onInitialize = function (scrippet) {};
            
            // This callback is to be called before HTML is closed
            this.onFinalize = function (scrippet) {};
            
            // This function is called from the Cocoa Application
            // to pass collection of snippets and variables into HTML
            this.initialize = function (sheet, snippetObjects, globalKeys, globalValues) {
                
                scrippetSheet = sheet;
                
                // Make a collection of snippets
                var numberOfSnippets = snippetObjects ? snippetObjects.length : 0;
                for (var i = 0; i < numberOfSnippets; i++)
                    snippets.push(new SnippetObject(snippetObjects[i]));
                
                // Make a collection of Scrippet properties
                var numberOfKeys = globalKeys ? globalKeys.length : 0
                for (var j = 0; j < numberOfKeys; j++)
                {
                    var key = globalKeys[j];
                    var value = globalValues[j];
                    if (key && value)
                        globalProperties[key] = value;
                }
                
                // Run callback after the data is loaded so user can setup UI
                try
                {
                    instance.onInitialize(this);
                }
                catch (error)
                {
                    alert("Exception on initialize: " + error);
                }
            };
            
            // This function is called from the Cocoa Application
            // to pass collection of snippets and variables back
            this.finalize = function () {
                
                // Run callback before HTML page is closed so user can collect the data
                try
                {
                    instance.onFinalize(this);
                }
                catch (error)
                {
                    alert("Exception on finalize: " + error);
                }
                
                // Collect key-value snippet dictionaries to pass into Cocoa
                var dictionaries = [];
                var numberOfSnippets = snippets.length;
                for (var i = 0; i < numberOfSnippets; i++)
                    dictionaries.push(snippets[i]._getCocoaDictionary());
                
                // The only way to pass dictionary back is to split it into keys and values
                var globalKeys = [];
                var globalValues = [];
                for (var key in globalProperties)
                {
                    globalKeys.push(key);
                    globalValues.push(globalProperties[key]);
                }
                
                scrippetSheet = null;
                
                // Pass collected data back to the Cocoa Application
                return {
                    snippets: dictionaries,
                    globalKeys: globalKeys,
                    globalValues: globalValues
                };
            };
            
            // This function is being called from the Cocoa side to
            // provide WebKit with a clipboard proxy object
            this.registerClipboardProxy = function (proxy) {
                clipboardProxy = proxy;
            };
            
            // This function uses clipboard proxy to set contents of
            // the Pasteboard from the Scrippet's WebKit page
            this.setClipboard = function (contents) {
                if (!clipboardProxy) return;
                clipboardProxy.setClipboard("" + contents);
            };
            
            // The following function allows to enable or disable
            // the Default button (usually named Continue or Close)
            this.setDefaultButtonEnabled = function (flag) {
                if (!scrippetSheet) return;
                if (flag)
                    scrippetSheet.setDefaultButtonEnabled(true);
                else
                    scrippetSheet.setDefaultButtonEnabled(false);
            }
            
            // Sometimes we need to continue scrippet by pressing the
            // Enter key in the form, for example. So, we can use
            // the following method to simulate pressing of default button
            this.clickDefaultButton = function () {
                if (!scrippetSheet) return;
                scrippetSheet.clickDefaultButton();
            };
            
            // Also sometimes we need to close scrippet sheet by pressing the
            // control in the form, for example. So, we can use
            // the following method to simulate pressing of alternate button
            this.clickAlternateButton = function () {
                if (!scrippetSheet) return;
                scrippetSheet.clickAlternateButton();
            };
        };
        
        // Return Scrippet singleton
        return new function() {
            if (instance == null) {
                instance = new ScrippetSingleton();
                instance.constructor = null;
            }
            return instance;
        }();
    }();
